@@ -4,8 +4,9 @@
 # file 'LICENSE', which is part of this source code package.
 #       Copyright (c) 2011 R Pratap Chakravarthy
 
-"""Instruction generator for :class:`StackMachine`. Generates the intermediate
-python file."""
+"""Instruction generator for :class:`tayra.runtime.StackMachine`. Generates
+intermediate python file.
+"""
 
 from   io   import   StringIO
 
@@ -13,8 +14,50 @@ import_header = """\
 import imp
 from   io                   import StringIO
 from   pluggdapps.plugin    import Plugin, implements
-import pluggdapps.utils     as h
-from   tayra                import BaseTTLPlugin"""
+from   tayra                import BaseTTLPlugin
+
+"""
+
+tb_decorator = """\
+def __traceback_decorator__( frames ):
+    from copy    import deepcopy
+    from os.path import basename
+
+    def _map2ttl( frame ):
+        filename = frame.filename
+        lineno = frame.lineno
+        lines = open(filename).readlines()[:lineno]
+        lines.reverse()
+        rc = {}
+        for l in lines :
+            if l.strip().startswith('# lineno') :
+                _, ttl_lineno = l.split(':', 1)
+                ttl_lineno = int( ttl_lineno )
+                ttl_text = open( _ttlfile ).readlines()[ ttl_lineno-1 ]
+                return ttl_lineno, ttl_text
+        return None, None
+
+    newframes = []
+    for frame in frames :
+        newframes.append( frame )
+        frameadded = getattr( frame, '_ttlframeadded', False )
+
+        basen = basename( frame.filename )
+        if basen.endswith( '.ttl.py' ) \
+             and basen == (basename( _ttlfile ) + '.py') \
+             and frameadded == False :
+            newframe = deepcopy( frame )
+            frame._ttlframeadded = True
+            try :
+                newframe.lineno, newframe.linetext = _map2ttl( newframe )
+                if newframe.lineno :
+                    newframe.filename = _ttlfile
+                    newframes.append( newframe )
+            except :
+                raise
+                continue
+    return newframes
+"""
 
 footer = """\
 _ttlhash = %r
@@ -35,7 +78,15 @@ class %s( BaseTTLPlugin ):
   implements(%s) 
 """
 
+
 class InstrGen( object ):
+    """Instruction generator to compile template script to an intermediate
+    python file. This file can be loaded as template module and executed in
+    the context of a HTTP request to generate the final HTML response. Since
+    creating an instance of this class every time can be a costly operation,
+    create it once and use :meth:`_init` to initialize for subsequent uses.
+    """
+
     machname = '_m'
 
     def __init__( self, compiler ):
@@ -54,19 +105,25 @@ class InstrGen( object ):
     #---- API
 
     def initialize( self ):
-        """Initialize and begin generating the intermediate python file."""
+        """Initialize and begin generating the intermediate python file.
+        Generates import statements and in debug mode generate trace-back
+        generator.
+        """
         self.outfd.write( import_header )
+        if self.compiler['debug'] :
+            self.outfd.write( tb_decorator )
         self.cr()
 
     def cr( self, count=1 ):
         """Generate a new-line (along with current indentation level) in
-        python generated text."""
+        python generated text.
+        """
         self.outfd.write( '\n'*count )
         self.outfd.write( self.pyindent )
 
     def codeindent( self, up=None, down=None, indent=True ):
-        """Increase or decrease the python code-indentation for the
-        intermediate text."""
+        """Increase or decrease python code-indentation for subsequently
+        generated code."""
         self.flushtext()
         if up != None :
             self.pyindent += up
@@ -113,14 +170,17 @@ class InstrGen( object ):
     def evalexprs( self, s ):
         """Evaluate a string as python expression and append the result into
         the stack buffer."""
-        try    : code, filts = s.split('|', 1)
-        except : code, filts = s, ''
-        code, filts = code.strip(), filts.strip()
-        if code :
+        name, s = s.split(' ',1) if s.startswith('-') else ('', s)
+        name = ('tayraexpression' + name[1:]) if name else name
+        try    : text, filts = s.split('|', 1)
+        except : text, filts = s, ''
+        text, filts = text.strip(), filts.strip()
+        if text :
             self.flushtext()
-            self.outline( 
-                '_m.append(_m.evalexprs( %r, %r, globals(), locals()) )' % (
-                        code, filts ))
+            self.outline(
+            '_m.append(_m.evalexprs( %r, %r, %r, globals(), locals()) )' % (
+                        name, text, filts )
+            )
 
     def pushbuf( self ):
         """Create a new buffer in the stack. The newly created buffer will
@@ -145,6 +205,17 @@ class InstrGen( object ):
         else :
             self.outline( 'return _m.popbuf()' )
 
+    def popobject( self, returnwith=None ):
+        """Some times a function might want to return something else other
+        than stacked buffers."""
+        self.flushtext()
+        if returnwith :
+            # Discard the newest buffer in the stack
+            self.outline( '_m.popbuf()' ) 
+            self.outline( 'return ' + returnwith )
+        else :
+            self.outline( 'return _m.popobject()' )
+
     def handletag( self, indent=False, newline='' ):
         """Pop the last two buffers from the stack and supply them to
         :class:`ITayraTags` plugins. The returned HTML text from the plugin is
@@ -160,7 +231,7 @@ class InstrGen( object ):
 
     def importttl( self, modname, ttlfile ):
         """Special method to handle @import directive."""
-        lines = self.importtext % (ttlfile, modname)
+        lines = importtext % (ttlfile, modname)
         self.putblock( lines )
 
     def putinherit( self, ttlloc ):
@@ -179,18 +250,23 @@ class InstrGen( object ):
             codeblock = interfaceClass % ( mod, ifname, pluginname, ifname )
             # hitch methods with interface class
             methods = interfaces_.get( ifname, [] )
-            methodlines = [ '  %s = %s' % (meth, meth) for meth in methods ]
+            methodlines = []
+            for meth in methods :
+                if meth == 'default_settings' :
+                    methodlines.append( '  %s = classmethod(%s)'%(meth, meth) )
+                else :
+                    methodlines.append( '  %s = %s' % (meth, meth) )
             self.putblock( '\n'.join( [codeblock] + methodlines ) )
 
-    def useinterface( self, module, interfacename, pluginname, name ):
-        line = 'from  %s import %s' % ( module, interfacename )
-        self.putstatement(line)
-        if isinstance(pluginname, tuple):
-            line = '%s = _m.use( %s, _m.evalexprs(%s, %r) )' % (
-                        name, interfacename, pluginname[0], pluginname[1] )
-        else :
-            line = '%s = _m.use( %s, %r )' % ( name, interfacename, pluginname )
-        self.putstatement(line)
+    #def useinterface( self, module, interfacename, pluginname, name ):
+    #    line = 'from  %s import %s' % ( module, interfacename )
+    #    self.putstatement(line)
+    #    if isinstance(pluginname, tuple):
+    #        line = '%s = _m.use( %s, _m.evalexprs(%s, %r) )' % (
+    #                    name, interfacename, pluginname[0], pluginname[1] )
+    #    else :
+    #        line = '%s = _m.use( %s, %r )' % (name, interfacename, pluginname)
+    #    self.putstatement(line)
 
     def footer( self, ttlhash, ttlfile ):
         """Add the footer python code."""
