@@ -6,6 +6,34 @@
 # file 'LICENSE', which is part of this source code package.
 #       Copyright (c) 2011 R Pratap Chakravarthy
 
+"""Command line interface to tayra package. When installing the package using
+easy_install, a shell command is automatically installed under **bin/**
+directory.
+
+.. code-block:: text
+
+  > tayra --help
+  usage: tayra [-h] [-l] [-d] [-s] [-t] [-a ARGS] [-c CONTEXT] [-g DEBUG]
+               [--version]
+               ttlfile
+
+  Pluggdapps command line script
+
+  positional arguments:
+    ttlfile     Input template file containing tayra script
+
+  optional arguments:
+    -h, --help  show this help message and exit
+    -l          Do lexical analysis of input file.
+    -d          Dump translation
+    -s          Show AST parse tree
+    -t          Execute test cases.
+    -a ARGS     Argument to template
+    -c CONTEXT  Context to template
+    -g DEBUG    Debug level for PLY argparser
+    --version   Version information of the package
+"""
+
 from   argparse            import ArgumentParser
 from   os.path             import isfile, join, dirname, basename
 import time, os
@@ -47,7 +75,7 @@ def mainoptions() :
             help='Version information of the package' )
     argparser.add_argument( 
             'ttlfile',
-            help='Output html file to store translated result' )
+            help='Input template file containing tayra script' )
 
     return argparser
 
@@ -85,7 +113,7 @@ def yaccer( pa, options, debuglevel=0 ):
     print( "Time taken to parse : ", time.time() - t1 )
     return ast
 
-if __name__ == '__main__' :
+def main() :
     argparser = mainoptions()
     options = argparser.parse_args()
     pa = Pluggdapps.boot( None )
@@ -104,6 +132,8 @@ if __name__ == '__main__' :
     elif options.test :
         from tayra.test.teststd import test_stdttl
         print( "Executing TTL tests ..." )
+        setts['beautify_html'] = True
+        compiler = pa.query_plugin(pa,ISettings,'ttlcompiler',settings=setts)
         test_stdttl( compiler, options )
 
     elif options.ttllex and options.ttlfile : 
@@ -127,5 +157,9 @@ if __name__ == '__main__' :
         ast.show()
 
     elif options.ttlfile and isfile( options.ttlfile ) :
+        print( "Translating file %r ..." % options.ttlfile )
         tayra.translatefile( options.ttlfile, compiler, options )
 
+
+if __name__ == '__main__' :
+    main()
